#version 130
#define float4 vec4
#define float3 vec3
#define float2 vec2
#define float4x4 mat4
#define DISCARD discard
#define lerp mix
#define tex2D texture
float4 mul(float4x4 mat, float4 v) { return mat * v; }
float saturate(float x) { return clamp(x,0.0,1.0); }

in float3 vertex;
in float3 normal;
#if USE_VERTEX_COLOURS==1
in float4 colour;
#if USE_NIGHT_COLOURS==1
in float4 uv1;
#endif
#endif
#if USE_TEXTURE==1
in vec2 uv0;
#endif

uniform float4x4 world;
uniform float4x4 view_proj;
uniform float4x4 shadow_world_view_proj1;
uniform float4x4 shadow_world_view_proj2;
uniform float4x4 shadow_world_view_proj3;
uniform float3 camera_pos;
uniform float4 light_pos; // 'w' is 1|0: tells if spot or directional light
uniform float4 light_pos_ws; // 'w' is 1|0: tells if spot or directional light
uniform float3 light_ambient;
uniform float3 surf_ambient;

#if USE_TEXTURE==1
out vec2 uv0_;
#endif
out float4 vertex_ls1_;
out float4 vertex_ls1_x_;
out float4 vertex_ls1_y_;
out float4 vertex_ls2_;
out float4 vertex_ls2_x_;
out float4 vertex_ls2_y_;
out float4 vertex_ls3_;
out float4 vertex_ls3_x_;
out float4 vertex_ls3_y_;
out float3 vertex_to_camera_;
out float3 vertex_to_light_;
out float3 normal_;
out float3 dyn_light_;
out float light_dist_;

void main(void)
{
        #if USE_VERTEX_COLOURS==1
        dyn_light_ = colour.rgb;
        gl_FrontColor.a = colour.a;
        #else
        dyn_light_ = float3(0,0,0);
        gl_FrontColor.a = 1;
        #endif

        // do ambient lighting
        gl_FrontColor.rgb = light_ambient * surf_ambient;

        // lighting vectors for fragment program
        vertex_to_light_ = normalize(light_pos.xyz - vertex*light_pos.w);
        vertex_to_camera_ = normalize(camera_pos - vertex);
        normal_ = normal;

        #if USE_TEXTURE==1
        uv0_ = uv0;
        #endif

        // vertex position in light space for shadow mapping
        vertex_ls1_ = mul(shadow_world_view_proj1, float4(vertex,1));
        vertex_ls2_ = mul(shadow_world_view_proj2, float4(vertex,1));
        vertex_ls3_ = mul(shadow_world_view_proj3, float4(vertex,1));

        float3 vertex_ws = mul(world, float4(vertex,1)).xyz;
        light_dist_ = -dot(vertex_ws,light_pos_ws.xyz);
 
        // for the rasteriser
        gl_Position = mul(view_proj, float4(vertex_ws,1));

}

// vim: ts=8:sw=8:expandtab
