print ("Initialising script...")
io.stdout:setvbuf("no") -- no output buffering
collectgarbage("setpause",110) -- begin a gc cycle after blah% increase in ram use
collectgarbage("setstepmul",150) -- collect at blah% the rate of new object creation

set_texture_verbose(false)
set_mesh_verbose(false)
get_main_win().autoDeactivate = false --don't disable rendering when focus lost
set_texture_budget(64*1024*1024)
set_mesh_budget(64*1024*1024)
fullScreenWidth = 800
fullScreenHeight = 600
windowedWidth = 800
windowedHeight = 600
--set_keyb_verbose(true)

include("system/strict.lua")

include("system/abbrev.lua")

include("system/util.lua")

add_resource_location(".","FileSystem",true)
initialise_all_resource_groups()

include("system/hud.lua")



print("Starting game engine...")


Main = {}

function Main:quit()
        self.shouldQuit = true
end

function Main.new()
        local self = {
                shouldQuit = false,
                frameCallbacks = CallbackReg.new()
        }

        return make_instance(self,Main)
end

function Main:run (...)

        -- execute cmdline arguments on console
        local arg = ""
        for i=2,select('#',...) do
                arg = arg.." "..select(i,...)
        end
        if #arg > 0 then
                console:exec(arg:sub(2))
        end

        local main_win = get_main_win()

        local last_focus = true

        local failName = {}

        -- rendering loop
        while not clicked_close() and not main.shouldQuit do

                
                if last_focus and not have_focus() then
                        keyb_flush() -- get rid of any sticky keys
                        last_focus = false
                        ui:updateGrabbed()
                end

                if not last_focus and have_focus() then
                        last_focus = true
                        ui:updateGrabbed()
                end


                xpcall(function ()

                        pump() -- handle OS GUI events (e.g. resizing, window close)
                        local t = Timer()

                        failName.name = nil
                        main.frameCallbacks:executeExtended(function (name,cb,...)
                                failName.name = name
                                --t:reset()
                                if cb == nil then
                                        --echo(RED.."Callback was nil: "..name)
                                        return true
                                end
                                local result = cb(...)
                                --local us = t.us
                                --if us>5000 and name~="GFX.frameCallback" then
                                --        print("callback \""..name.."\" took "..us/1000 .."ms ")
                                --end
                                return result
                        end)
                        failName.name = nil

                        if main_win.isActive == false then
                                sleep(3E7)
                        end

                end,error_handler)

                if failName.name then
                        echo("Removed frameCallback: "..failName.name)
                        main.frameCallbacks:removeByName(failName.name)
                end

        end
        
end





main = Main.new()


include("system/ui.lua")

include("system/anaglyph_camera.lua")

include("system/gfx.lua")

include("system/physics.lua")

include("system/console_prompt.lua")
include("system/console.lua")

include("system/player_ctrl.lua")

include("system/world.lua")

include("system/capturer.lua")


include("common/init.lua")

include("vehicles/init.lua")


ui:bind("F5",function()
        local pm = gfx.cam.polygonMode
        if pm == "SOLID" then
                gfx.cam.polygonMode = "WIREFRAME"
        elseif pm == "WIREFRAME" then
                gfx.cam.polygonMode = "POINTS"
        else
                gfx.cam.polygonMode = "SOLID"
        end
end)
ui:bind("F6",function()
        gfx.scnmgr.showBoundingBoxes = not gfx.scnmgr.showBoundingBoxes
end)
-- don't use bind so that we don't get in the way of other stuff that uses Return
ui.pressCallbacks:insert("Main.fullscreenCallback",function (key)
        if key=="+Return" and ui:alt() then
                local win = get_main_win()
                local b = win.isFullScreen
                if b then
                        win:setFullScreen(false,windowedWidth,windowedHeight)
                else
                        win:setFullScreen(true,fullScreenWidth,fullScreenHeight)
                end
                keyb_flush("Alt")
                keyb_flush("Return")
                ui:flush("Alt")
                ui:flush("Return")
                return false
        end
end,1)
ui:bind("c", function() if ui:ctrl() then main:quit() end end)

function reload_depth_shaders()
        get_gpuprog("common/depth_v"):reload()
        get_gpuprog("common/depth_f"):reload()
end


-- vim: shiftwidth=8:tabstop=8:expandtab
