print("Loading player_ctrl.lua")

PlayerCtrl = { } 

-- reuse object to avoid allocations
local displacement = Vector3(0,0,0)

function PlayerCtrl:strafe()
        local right = self.right - self.left
        local forwards = self.forwards - self.backwards
        local up = self.up - self.down

        local interval = self.frameTimer.us/1000000
        self.frameTimer:reset()
        

        local active_speed = self.fast and self.speedFast or self.speedSlow

        local dist = active_speed * interval

        -- we now know how far to move (dist)
        -- but not in which directions

        displacement:xyz(dist*right, dist*forwards, 0)
        local pain = gfx.camRoll.derivedOrientation
        displacement:rotate(pain)
        
        displacement:append(0, 0, dist*up)

        local fraction, _, normal = physics.world:cast(0.15, gfx.camPos.derivedPosition,
                                                       gfx.camPos.derivedPosition + displacement,
                                                       0, true, true)

        if fraction ~= nil then
                normal = normal.normalised
                displacement = fraction * displacement + ((1-fraction) * displacement - normal * (displacement:dot(normal)))
                local _, fraction2, _ =
                        physics.world:cast(0.15, gfx.camPos.derivedPosition,
                                           gfx.camPos.derivedPosition + displacement,
                                           0, true, true)
                if fraction2 ~= nil then
                        return
                end
        end
        
        -- splendid, now let's move

        gfx.camPos:translate(displacement)

end

function PlayerCtrl:pickDrive()
        local obj = world:pickObjSafe()
        if obj == nil then return end
        self:drive(obj)
end

function PlayerCtrl:drive(vehicle)
        if not vehicle.activated then error("Can't drive a vehicle that isn't activated") end
        self.vehicle = vehicle
        local yaw = gfx:yaw()
        gfx.cam:detach()
        gfx.cam:attach(gfx.camChaseBoom)
        if gfx.camChase.parentSceneNode then
                gfx.camChase.parentSceneNode:removeChild(gfx.camChase)
        end
        vehicle.instance.gfx:addChild(gfx.camChase)
        gfx.camChaseYaw.orientation = yaw
        gfx.camChasePitch.orientation = gfx.camPitch.orientation
        gfx.camChaseRoll.orientation = Q_ID
        self.mode = 1
end

function PlayerCtrl:abandonVehicle()
        if self.vehicle and self.vehicle.activated and self.vehicle.abandon ~= nil then
                self.vehicle:abandon()
        end
        self.vehicle = nil
        local pos = gfx.cam.derivedPosition
        local yaw = gfx:yaw()
        gfx.cam:detach()
        gfx.cam:attach(gfx.camRoll)
        gfx.camPos.position = pos
        gfx.camYaw.orientation = yaw
        gfx.camPitch.orientation = gfx.camChasePitch.orientation
        self.mode = 0
        self.speedo.text = "Ghost Mode"
end

function PlayerCtrl:checkBoomLength()
        local v = self.vehicle
        if v==nil then return end
        if not v.activated then return end
        local body = v.instance.body
        local ray_reach =
                body.world:cast(self.boomRayRadius,
                          gfx.camChaseRoll.derivedPosition,
                          gfx.camChaseBoom.derivedOrientation * V_BACKWARDS,
                          self.currentBoomLength, true, true)

        ray_reach = ray_reach or self.currentBoomLength

        gfx.camChaseBoom.position = ray_reach * V_BACKWARDS
end

function PlayerCtrl:zoom()
        if ui:shift() then
                self:zoomOut()
        else
                self:zoomIn()
        end
end

function PlayerCtrl:zoomIn()
        self.currentBoomLength = clamp(self.currentBoomLength - 0.2,0,40)
end

function PlayerCtrl:zoomOut()
        self.currentBoomLength = clamp(self.currentBoomLength + 0.2,0,40)
end

function PlayerCtrl.new()

        local self = {
                fast = false,
                frameTimer = Timer(),

                mouseSensitivity = 0.06,
                mouseInvert = true,

                speedSlow = 25,
                speedFast = 300,

                forwards = 0,
                backwards = 0,
                left = 0,
                right = 0,
                up = 0,
                down = 0,

                currentBoomLength = gfx.camChaseBoom.position.y,
                boomRayRadius = 0.1,

                mode = 0, -- modes are 0 (ghost)  1 (drive)
                ghostBinds = BindTable.new(),
                walkBinds = BindTable.new(),
                driveBinds = BindTable.new(),
        }

        local opt = {font = "misc.fixed", charHeight = 13}
        self.speedo = get_hud_root():addChild("ShadowText",opt)
        self.speedo.resize = Hud.BOTTOM_LEFT()
        self.speedo.text = "Ghost Mode"

        local ghostBinds, driveBinds = self.ghostBinds, self.driveBinds

        ghostBinds:bind("w", function() self.forwards=1 end, function() self.forwards = 0 end)
        ghostBinds:bind("s", function() self.backwards=1 end, function() self.backwards = 0 end)
        ghostBinds:bind("a", function() self.left=1 end, function() self.left = 0 end)
        ghostBinds:bind("d", function() self.right=1 end, function() self.right = 0 end)
        ghostBinds:bind("f", function() self:pickDrive() end)
        ghostBinds:bind("Space", function() self.up=1 end, function() self.up = 0 end)
        ghostBinds:bind("Shift", function() self.down=1 end, function() self.down = 0 end)
        ghostBinds:bind("right", function() self.fast = true end, function() self.fast = false end)

        driveBinds:bind("w", function() self.vehicle:setPush(true) end,
                             function() self.vehicle:setPush(false) end)
        driveBinds:bind("s", function() self.vehicle:setPull(true) end,
                             function() self.vehicle:setPull(false) end)
        driveBinds:bind("a", function() self.vehicle:setShouldSteerLeft(true) end,
                             function() self.vehicle:setShouldSteerLeft(false) end)
        driveBinds:bind("d", function() self.vehicle:setShouldSteerRight(true) end,
                             function() self.vehicle:setShouldSteerRight(false) end)
        driveBinds:bind("Space", function() self.vehicle:setHandbrake(true) end,
                                 function() self.vehicle:setHandbrake(false) end)
        driveBinds:bind("Return", function() self.vehicle:realign() end)
        driveBinds:bind("f", function() self:abandonVehicle() end)
        driveBinds:bind("left", function() self.vehicle:boost() end)
        driveBinds:bind("v", function() self:zoom() end, nil, true)
        driveBinds:bind("up", function() self:zoomIn() end)
        driveBinds:bind("down", function() self:zoomOut() end)

        ui.pressCallbacks:insert("PlayerCtrl",function (key)
                if self.mode == 0 then
                        return ghostBinds:process(key)
                elseif self.mode == 1 then
                        return driveBinds:process(key)
                else
                        error("PlayerCtrl.mode unrecognised! ("..self.mode..")")
                end
        end)

        ui.pointerGrabCallbacks:insert("PlayerCtrl",function (rel_x,rel_y)
                local yaw, pitch

                if self.mode == 0 then
                        yaw, pitch = gfx.camYaw, gfx.camPitch
                elseif self.mode == 1 then
                        yaw, pitch = gfx.camChaseYaw, gfx.camChasePitch
                end

                local sens = self.mouseSensitivity
                local inv = self.mouseInvert and 1 or -1
                local rot_x, rot_y = rel_x*sens, inv*rel_y*sens

                yaw.orientation   = yaw.orientation   * Quat(rot_x,V_DOWN)
                local p = pitch.orientation * Quat(rot_y,V_RIGHT)

                if p.angle > 90 then
                        local sign = p.x
                        local v = Vector3(sign>0 and 1 or -1, 0, 0)
                        p = Quat(90,v)
                end
                pitch.orientation = p
        end)

        main.frameCallbacks:insert("PlayerCtrl.frameCallback", function ()
                if self.mode == 0 then
                        self:strafe()
                elseif self.mode == 1 then
                        self:checkBoomLength()
                        if self.vehicle.activated == false then
                                self:abandonVehicle()
                                return
                        end
                        if self.vehicle.getSpeed ~= nil then
                                if self.speedo ~= nil then
                                        self.speedo.text =
                                                string.format("Speed: %d MPH\n",
                                                              self.vehicle:getSpeed()*60*60/1500)
                                end
                        end
                end
        end)

        return make_instance(self,PlayerCtrl)
end

function PlayerCtrl:flush()
        if self.mode == 0 then
                self.ghostBinds:flush()
        elseif self.mode == 1 then
                self.driveBinds:flush()
        else
                error("PlayerCtrl.mode unrecognised! ("..self.mode..")")
        end
end

function PlayerCtrl:destroy()
        main.frameCallbacks:removeByName("PlayerCtrl.frameCallback")
        ui.pressCallbacks:removeByName("PlayerCtrl")
        ui.pointerGrabCallbacks:removeByName("PlayerCtrl")
        self.speedo = safe_destroy(self.speedo)
end

if player_ctrl ~= nil then
        player_ctrl:destroy()
end
player_ctrl = PlayerCtrl.new()

-- vim: shiftwidth=8:tabstop=8:expandtab
